@@ -107,54 +107,3 @@ gstreamer_test_pipeline(gchar *pipeline)
     
     return error == NULL;
 }
-
-static void
-gst_typefind_type_found_callback(GstElement *typefind, guint probability, 
-    GstCaps *caps, gchar **type)
-{
-    *type = gst_caps_to_string(caps);    
-}
-
-gchar *
-gstreamer_detect_mimetype(const gchar *uri)
-{
-    return NULL;
-    /*GstElement *pipeline;
-    GstElement *source;
-    GstElement *typefind;
-    GstElement *fakesink;
-    gchar *mimetype = NULL;
-
-    pipeline = gst_pipeline_new("new");
-        
-    source = gst_element_factory_make("gnomevfssrc", "source");
-    typefind = gst_element_factory_make("typefind", "typefind");
-    fakesink = gst_element_factory_make("fakesink", "fakesink");
-
-    if(source == NULL || typefind == NULL) {
-        gst_object_unref(pipeline);
-        return NULL;
-    }
-
-    g_object_set(source, "location", uri, NULL);
-    g_signal_connect(typefind, "have-type", 
-        G_CALLBACK(gst_typefind_type_found_callback), &mimetype);
-
-    gst_bin_add_many(GST_BIN(pipeline), source, typefind, fakesink, NULL);
-    gst_element_link(source, typefind);
-    gst_element_link(typefind, fakesink);
-    
-    if(gst_element_set_state(pipeline, GST_STATE_PLAYING) == GST_STATE_CHANGE_ASYNC) {
-        if (gst_element_get_state(pipeline, NULL, NULL, -1) == GST_STATE_CHANGE_FAILURE) {
-            gst_object_unref(pipeline);
-            return NULL;
-        }
-    }
-
-    while((mimetype == NULL) && (pipeline->current_state == GST_STATE_PLAYING));
-    
-    gst_element_set_state(pipeline, GST_STATE_NULL);
-    gst_object_unref(pipeline);
-        
-    return mimetype;*/
-}
