@@ -93,8 +93,8 @@ namespace Banshee.Base
                     Catalog.GetString("Import source to library"), null),
                     
                 new ActionEntry("SelectedSourcePropertiesAction", Stock.Properties,
-                    Catalog.GetString("Source Properties..."), null,
-                    Catalog.GetString("View source properties"), null),
+                    "Source Properties", null,
+                    null, null),
                     
                 new ActionEntry("QuitAction", Stock.Quit,
                     Catalog.GetString("_Quit"), "<control>Q",
@@ -211,15 +211,19 @@ namespace Banshee.Base
             /* Song Selected Actions */
             
             song_actions.Add(new ActionEntry [] {
+                new ActionEntry("CopySongsAction", Stock.Copy,
+                    Catalog.GetString("Copy"), "<Control>C",
+                    Catalog.GetString("Copy selected song(s) to clipboard"), null),
+                    
                 new ActionEntry("RemoveSongsAction", Stock.Remove,
-                    Catalog.GetString("Remove Song(s)"), "Delete",
+                    Catalog.GetString("Remove"), "Delete",
                     Catalog.GetString("Remove selected song(s) from library"), null),
                     
                 new ActionEntry("DeleteSongsFromDriveAction", null,
-                    Catalog.GetString("Delete Song(s) From Drive"), null,
+                    Catalog.GetString("Delete From Drive"), null,
                     Catalog.GetString("Permanently delete selected song(s) from storage medium"), null),
                     
-                new ActionEntry("PropertiesAction", Stock.Properties,
+                new ActionEntry("PropertiesAction", Stock.Edit,
                     Catalog.GetString("Edit Song Metadata"), null,
                     Catalog.GetString("Edit metadata on selected songs"), null),
 
