@@ -55,6 +55,11 @@ namespace Banshee.Base
         public static event ShutdownRequestHandler ShutdownRequested;
         
         public static void Initialize()
+        {
+            Initialize(null);
+        }
+        
+        public static void Initialize(ComponentInitializerHandler interfaceStartupHandler)
         {
             if(!Directory.Exists(Paths.ApplicationData)) {
                 Directory.CreateDirectory(Paths.ApplicationData);
@@ -88,6 +93,7 @@ namespace Banshee.Base
                 Catalog.GetString("Device support will be disabled for this instance (no HAL)"),
                 HalCore.Initialize);
                 
+            startup.Register(Catalog.GetString("Initializing audio engine"), Banshee.Gstreamer.Utilities.Initialize);
             startup.Register(Catalog.GetString("Starting background tasks"), delegate { gconf_client = new GConf.Client(); });
             startup.Register(Catalog.GetString("Detecting network settings"), delegate { network_detect = NetworkDetect.Instance; });
             startup.Register(Catalog.GetString("Creating action manager"), delegate { action_manager = new ActionManager(); });
@@ -96,9 +102,7 @@ namespace Banshee.Base
                 library.ReloadLibrary();
             });
             
-            startup.Register(Catalog.GetString("Initializing audio"), Banshee.Gstreamer.Utilities.Initialize);
-            
-            startup.Register(Catalog.GetString("Initializing audio"), delegate {
+            startup.Register(Catalog.GetString("Initializing audio profiles"), delegate {
                 string system_path = Path.Combine(Banshee.Base.Paths.SystemApplicationData, "audio-profiles");
                 string user_path = Path.Combine(Banshee.Base.Paths.ApplicationData, "audio-profiles");
                 string env_path = Environment.GetEnvironmentVariable("BANSHEE_PROFILES_PATH");
@@ -127,7 +131,7 @@ namespace Banshee.Base
                 audio_profile_manager.TestAll();
             });
             
-            startup.Register(Catalog.GetString("Initializing audio"), PlayerEngineCore.Initialize);
+            startup.Register(Catalog.GetString("Initializing audio engine"), PlayerEngineCore.Initialize);
             
             startup.Register(Catalog.GetString("Initializing audio CD support"), true, 
                 Catalog.GetString("Audio CD support will be disabled for this instance"), delegate { audio_cd_core = new AudioCdCore(); });
@@ -141,9 +145,11 @@ namespace Banshee.Base
             startup.Register(Catalog.GetString("Initializing plugins"), Banshee.Plugins.PluginCore.Initialize);
             startup.Register(Catalog.GetString("Starting background tasks"), PowerManagement.Initialize);
             
-            startup.Run();
+            if(interfaceStartupHandler != null) {
+                startup.Register(Catalog.GetString("Loading user interface"), interfaceStartupHandler);
+            }
             
-            action_manager.LoadInterface();
+            startup.Run();
         }
         
         public static void Shutdown()
@@ -252,9 +258,7 @@ namespace Banshee.Base
         public static Gtk.Window MainWindow {
             get { return main_window; }
             set {
-                if(main_window == null) {
-                    main_window = value;
-                }
+                main_window = value;
             }
         }
 
