@@ -57,7 +57,6 @@ namespace Banshee.Sources
               
             Globals.Library.TrackAdded += delegate(object o, LibraryTrackAddedArgs args) {
                 OnTrackAdded(args.Track);
-                OnUpdated();
             };  
 
             SortCriteria criteria;
@@ -101,6 +100,33 @@ namespace Banshee.Sources
         {
             Globals.Library.CommitRemoveQueue();
         }
+        
+        private Queue<TrackInfo> add_pending_queue = new Queue<TrackInfo>();
+        private uint add_pending_queue_flush_timeout = 0;
+        
+        public override void OnTrackAdded(TrackInfo track)
+        {
+            lock(this) {
+                add_pending_queue.Enqueue(track);
+            
+                if(add_pending_queue_flush_timeout == 0) {
+                    add_pending_queue_flush_timeout = GLib.Timeout.Add(1500, FlushAddPendingQueue);
+                }
+            }
+        }
+        
+        private bool FlushAddPendingQueue()
+        {
+            lock(this) {
+                while(add_pending_queue.Count > 0) {
+                    base.OnTrackAdded(add_pending_queue.Dequeue());
+                    base.OnUpdated();
+                }
+                
+                add_pending_queue_flush_timeout = 0;
+                return false;
+            }
+        }
 
         private Gtk.ActionGroup action_group = null;
         public override string ActionPath {
@@ -162,6 +188,42 @@ namespace Banshee.Sources
             get { return Globals.Library.Tracks.Count; }
         }  
         
+        public override int SortColumn {
+            get { 
+                try {
+                    return (int)Globals.Configuration.Get(GConfKeys.LibrarySortColumn);
+                } catch {
+                    return base.SortColumn;
+                }
+            }
+            
+            set {
+                try {
+                    Globals.Configuration.Set(GConfKeys.LibrarySortColumn, value);
+                } catch {
+                    base.SortColumn = value;
+                }
+            }
+        }        
+        
+        public override Gtk.SortType SortType {
+            get { 
+                try {
+                    return (Gtk.SortType)Globals.Configuration.Get(GConfKeys.LibrarySortType);
+                } catch {
+                    return base.SortType;
+                }
+            }
+            
+            set {
+                try {
+                    Globals.Configuration.Set(GConfKeys.LibrarySortType, (int)value);
+                } catch {
+                    base.SortType = value;
+                }
+            }
+        }
+        
         private static Gdk.Pixbuf icon = IconThemeUtils.LoadIcon(22, Gtk.Stock.Home, "user-home", "source-library");
         public override Gdk.Pixbuf Icon {
             get { return icon; } 
