@@ -108,7 +108,7 @@ namespace Banshee.Sources
         {
             Id = Globals.Library.Db.Execute(new DbCommand(
                 @"INSERT INTO Playlists
-                    VALUES (NULL, :playlist_name)",
+                    VALUES (NULL, :playlist_name, -1, 0)",
                     "playlist_name", Name
             ));
         }
@@ -232,6 +232,10 @@ namespace Banshee.Sources
         
         public override bool Unmap()
         {
+            if(Count > 0 && !PlaylistUtil.ConfirmUnmap(this)) {
+                return false;
+            }
+        
             Globals.Library.Db.Execute(new DbCommand(
                 @"DELETE FROM PlaylistEntries
                     WHERE PlaylistID = :playlist_id",
@@ -395,6 +399,60 @@ namespace Banshee.Sources
         
         public override bool IsDragSource {
             get { return true; }
+        }        
+        
+        public override int SortColumn {
+            get { 
+                try {
+                    return Convert.ToInt32(Globals.Library.Db.QuerySingle(new DbCommand(
+                        @"SELECT SortColumn
+                            FROM Playlists
+                            WHERE PlaylistID = :playlist_id",
+                            playlist_id_param)));
+                } catch {
+                    return base.SortColumn;
+                }
+            }
+            
+            set {
+                try {
+                    Globals.Library.Db.Execute(new DbCommand(
+                        @"UPDATE Playlists
+                            SET SortColumn = :sort_column
+                            WHERE PlaylistID = :playlist_id",
+                            "sort_column", value,
+                            playlist_id_param));
+                } catch {
+                    base.SortColumn = value;
+                }
+            }
+        }        
+        
+        public override Gtk.SortType SortType {
+            get { 
+                try {
+                    return (Gtk.SortType)Convert.ToInt32(Globals.Library.Db.QuerySingle(new DbCommand(
+                        @"SELECT SortType
+                            FROM Playlists
+                            WHERE PlaylistID = :playlist_id",
+                            playlist_id_param)));
+                } catch {
+                    return base.SortType;
+                }
+            }
+            
+            set {
+                try {
+                    Globals.Library.Db.Execute(new DbCommand(
+                        @"UPDATE Playlists
+                            SET SortType = :sort_type
+                            WHERE PlaylistID = :playlist_id",
+                            "sort_type", value,
+                            playlist_id_param));
+                } catch {
+                    base.SortType = value;
+                }
+            }
         }
         
         private static Gdk.Pixbuf icon = IconThemeUtils.LoadIcon(22, "source-playlist");
@@ -450,5 +508,53 @@ namespace Banshee.Sources
             return NamingUtil.PostfixDuplicate(NamingUtil.GenerateTrackCollectionName(
                 tracks, Catalog.GetString("New Playlist")), PlaylistExists);
         }
+        
+        public static bool ConfirmUnmap(Source source)
+        {
+            bool do_not_ask = false;
+            string key = GConfKeys.BasePath + "no_confirm_unmap_" + source.GetType().Name.ToLower();
+            
+            try {
+                do_not_ask = (bool)Globals.Configuration.Get(key);
+            } catch {
+            }
+            
+            if(do_not_ask) {
+                return true;
+            }
+        
+            Banshee.Widgets.HigMessageDialog dialog = new Banshee.Widgets.HigMessageDialog(
+                InterfaceElements.MainWindow,
+                Gtk.DialogFlags.Modal,
+                Gtk.MessageType.Question,
+                Gtk.ButtonsType.Cancel,
+                String.Format(Catalog.GetString("Are you sure you want to delete this {0}?"),
+                    source.GenericName.ToLower()),
+                source.Name);
+            
+            dialog.AddButton(Gtk.Stock.Delete, Gtk.ResponseType.Ok, false);
+            
+            Gtk.Alignment alignment = new Gtk.Alignment(0.0f, 0.0f, 0.0f, 0.0f);
+            alignment.TopPadding = 10;
+            Gtk.CheckButton confirm_button = new Gtk.CheckButton(String.Format(Catalog.GetString(
+                "Do not ask me this again"), source.GenericName.ToLower()));
+            confirm_button.Toggled += delegate {
+                do_not_ask = confirm_button.Active;
+            };
+            alignment.Add(confirm_button);
+            alignment.ShowAll();
+            dialog.LabelVBox.PackStart(alignment, false, false, 0);
+            
+            try {
+                if(dialog.Run() == (int)Gtk.ResponseType.Ok) {
+                    Globals.Configuration.Set(key, do_not_ask);
+                    return true;
+                }
+                
+                return false;
+            } finally {
+                dialog.Destroy();
+            }
+        }
     }
 }
