@@ -40,6 +40,16 @@ using GConf;
 using Banshee.MediaEngine;
 using Banshee.Base;
 
+public static class PluginModuleEntry
+{
+    public static Type [] GetTypes()
+    {
+        return new Type [] {
+            typeof(Banshee.Plugins.Audioscrobbler.AudioscrobblerPlugin)
+        };
+    }
+}
+
 namespace Banshee.Plugins.Audioscrobbler 
 {
     public class AudioscrobblerPlugin : Banshee.Plugins.Plugin
@@ -176,7 +186,7 @@ namespace Banshee.Plugins.Audioscrobbler
         private void StartEngine()
         {
             if(protocol_engine == null) {
-                Console.WriteLine("Audioscrobbler starting protocol engine");
+                LogCore.Instance.PushDebug("Audioscrobbler starting protocol engine", "");
                 protocol_engine = new Engine();
                 protocol_engine.SetUserPassword(Username, Password);
         			protocol_engine.Start();
@@ -186,7 +196,7 @@ namespace Banshee.Plugins.Audioscrobbler
         private void StopEngine()
         {
             if(protocol_engine != null) {
-                Console.WriteLine("Audioscrobbler stopping protocol engine");
+                LogCore.Instance.PushDebug("Audioscrobbler stopping protocol engine", "");
                 protocol_engine.Stop();
                 protocol_engine = null;
             }
