@@ -34,11 +34,23 @@ using System;
 using Gtk;
 using GLib;
 using GConf;
+using Mono.Unix;
+
+using Notifications;
+
 using Banshee.Base;
 using Banshee.MediaEngine;
 using Banshee.Widgets;
-using Mono.Unix;
-using Notifications;
+
+public static class PluginModuleEntry
+{
+    public static Type [] GetTypes()
+    {
+        return new Type [] {
+            typeof(Banshee.Plugins.NotificationAreaIcon.NotificationAreaIconPlugin)
+        };
+    }
+}
 
 namespace Banshee.Plugins.NotificationAreaIcon 
 {
@@ -64,6 +76,7 @@ namespace Banshee.Plugins.NotificationAreaIcon
         private EventBox event_box;
         private NotificationArea notif_area;
         private Menu menu;
+        private ActionGroup actions;
         private uint ui_manager_id;
 
         private TrackInfoPopup popup;
@@ -75,11 +88,27 @@ namespace Banshee.Plugins.NotificationAreaIcon
         private static readonly uint SkipDelta = 10;
         private static readonly int VolumeDelta = 10;
         
-        protected override void PluginInitialize() 
+        protected override void PluginInitialize()
+        {
+        }
+        
+        protected override void InterfaceInitialize() 
         {
             Init();
 
-            ui_manager_id = Globals.ActionManager.UI.AddUiFromResource("NotificationAreaIconMenu.xml");        
+            actions = new ActionGroup("NotificationArea");
+
+            actions.Add(new ActionEntry [] {
+                new ActionEntry("CloseAction", Stock.Close,
+                    Catalog.GetString("Close"), "<Control>W",
+                    Catalog.GetString("Close"), delegate { 
+                        ShowHideMainWindow(); 
+                 })
+            });
+
+            Globals.ActionManager.UI.InsertActionGroup(actions, 0);
+            ui_manager_id = Globals.ActionManager.UI.AddUiFromResource("NotificationAreaIconMenu.xml");      
+            
             menu = (Menu) Globals.ActionManager.UI.GetWidget("/NotificationAreaIconMenu");
             menu.ShowAll();
         
@@ -93,6 +122,7 @@ namespace Banshee.Plugins.NotificationAreaIcon
 
             // Forcefully load this value
             show_notifications = ShowNotifications;
+            InterfaceElements.MainWindow.KeyPressEvent += OnKeyPressEvent;
         }
 
         protected override void PluginDispose() 
@@ -102,20 +132,18 @@ namespace Banshee.Plugins.NotificationAreaIcon
                 event_box = null;
                 notif_area = null;
             }
+            
             PlayerEngineCore.EventChanged -= OnPlayerEngineEventChanged;
+            
             Globals.ActionManager.UI.RemoveUi(ui_manager_id);
+            Globals.ActionManager.UI.RemoveActionGroup(actions);
         }
 
         public override Gtk.Widget GetConfigurationWidget()
         {            
             return new NotificationAreaIconConfigPage(this);
         }
-
-        protected override void InterfaceInitialize() 
-        {
-            InterfaceElements.MainWindow.KeyPressEvent += OnKeyPressEvent;
-        }
-
+        
         private void Init() 
         {
             notif_area = new NotificationArea(Catalog.GetString("Banshee"));
@@ -141,15 +169,17 @@ namespace Banshee.Plugins.NotificationAreaIcon
         private void ShowHideMainWindow()
         {
             if (InterfaceElements.MainWindow.IsActive) {
+                SaveWindowSizePosition();
                 InterfaceElements.MainWindow.Visible = false;
             } else {
+                RestoreWindowSizePosition();
                 InterfaceElements.MainWindow.Present();
             }
         }
 
         private void ShowNotification()
         {
-            if (cursor_over_trayicon || !show_notifications) {
+            if (cursor_over_trayicon || !show_notifications || InterfaceElements.MainWindow.HasToplevelFocus) {
                 return;
             }
             
@@ -183,7 +213,6 @@ namespace Banshee.Plugins.NotificationAreaIcon
             if (args.Event.Key == Gdk.Key.w && (args.Event.State & Gdk.ModifierType.ControlMask) != 0) {
                 handled = true;
                 ShowHideMainWindow();
-                ResizeMoveWindow();
             }
             
             args.RetVal = handled;
@@ -201,7 +230,6 @@ namespace Banshee.Plugins.NotificationAreaIcon
                         Globals.ActionManager["PreviousAction"].Activate();
                     } else {
                         ShowHideMainWindow();
-                        ResizeMoveWindow();
                     }
                     break;
                 case 2:
@@ -362,43 +390,27 @@ namespace Banshee.Plugins.NotificationAreaIcon
             }
         }
 
-        //FIXME: GO AWAY UGLY COPY!!!
-        private void ResizeMoveWindow() {
-            Window WindowPlayer = InterfaceElements.MainWindow;
-            int x = 0, y = 0, width = 0, height = 0;
-            try {
-                x = (int)Globals.Configuration.Get(GConfKeys.WindowX);
-                y = (int)Globals.Configuration.Get(GConfKeys.WindowY);
-                
-                width = (int)Globals.Configuration.Get(GConfKeys.WindowWidth);
-                height = (int)Globals.Configuration.Get(GConfKeys.WindowHeight);
-            } catch(GConf.NoSuchKeyException) {
-                width = 800;
-                height = 600;
-                x = 0;
-                y = 0;
-            }
-            
-            if(width != 0 && height != 0) {
-                WindowPlayer.Resize(width, height);
-            }
-            
-            if(x == 0 && y == 0) {
-                WindowPlayer.SetPosition(Gtk.WindowPosition.Center);
-            } else {
-                WindowPlayer.Move(x, y);
-            }
-            
-            try {
-                if((bool)Globals.Configuration.Get(GConfKeys.WindowMaximized)) {
-                    WindowPlayer.Maximize();
-                } else {
-                    WindowPlayer.Unmaximize();
-                }
-            } catch(GConf.NoSuchKeyException) {
+        private int x, y, w, h;
+        private bool maximized;
+        private void SaveWindowSizePosition()
+        {
+            maximized = ((InterfaceElements.MainWindow.GdkWindow.State & Gdk.WindowState.Maximized) > 0);
+
+            if (!maximized) {
+                InterfaceElements.MainWindow.GetPosition(out x, out y);
+                InterfaceElements.MainWindow.GetSize(out w, out h);
             }
         }
-        
+
+        private void RestoreWindowSizePosition() {
+            if (maximized) {
+                InterfaceElements.MainWindow.Maximize();
+            } else {
+                InterfaceElements.MainWindow.Resize(w, h);
+                InterfaceElements.MainWindow.Move(x, y);
+            }
+        }
+
         public bool ShowNotifications {
             get { 
                 try {
