@@ -28,7 +28,7 @@
 
 using System;
 using System.IO;
-
+using System.Text;
 using System.Threading;
 using System.Reflection;
 using System.Collections;
@@ -120,6 +120,11 @@ namespace Banshee
                 Banshee.Gui.DragDrop.DragDropTarget.PlaylistRows,
                 Banshee.Gui.DragDrop.DragDropTarget.UriList
             };
+            
+        private static TargetEntry [] nautilus_file_copy_entries = 
+            new TargetEntry [] { 
+                new TargetEntry("x-special/gnome-copied-files", 0, 0)
+            };
 
         public PlayerUI() 
         {
@@ -127,6 +132,7 @@ namespace Banshee
             gxml.Autoconnect(this);
             InterfaceElements.MainWindow = WindowPlayer;
 
+            Globals.ActionManager.LoadInterface();
             ResizeMoveWindow();
             BuildWindow();   
             
@@ -141,15 +147,13 @@ namespace Banshee
             Globals.DBusPlayer.UIAction += OnDBusPlayerUIAction;
             
             InitialLoadTimeout();
-            if(!Globals.ArgumentQueue.Contains("hide")) {
-                WindowPlayer.Show();
-            }
             
             // Bind available methods to actions defined in ActionManager
             Globals.ActionManager.DapActions.Visible = false;
             Globals.ActionManager.AudioCdActions.Visible = false;
             Globals.ActionManager.SongActions.Sensitive = false;
             Globals.ActionManager.PlaylistActions.Sensitive = false;
+            Globals.ActionManager["JumpToPlayingAction"].Visible = false;
             
             foreach(Action action in Globals.ActionManager) {
                 string method_name = "On" + action.Name;
@@ -174,6 +178,10 @@ namespace Banshee
             
             Globals.UIManager.SourceViewContainer = gxml["SourceViewContainer"] as Box;
             Globals.UIManager.Initialize();
+            
+            if(!Globals.ArgumentQueue.Contains("hide")) {
+                WindowPlayer.Show();
+            }
         }
    
         private bool InitialLoadTimeout()
@@ -377,24 +385,23 @@ namespace Banshee
             (gxml["LeftContainer"] as Box).PackStart(cover_art_view, false, false, 0);
             
             // Source View
-            Label sourceViewLoading = new Label();
-            sourceViewLoading.Yalign = 0.15f;
-            sourceViewLoading.Xalign = 0.5f;
-            sourceViewLoading.Markup = "<big><i>" + Catalog.GetString("Loading...") + "</i></big>";
-            sourceViewLoadingVP = new Viewport();
-            sourceViewLoadingVP.ShadowType = ShadowType.None;
-            sourceViewLoadingVP.Add(sourceViewLoading);
-            sourceViewLoadingVP.ShowAll();
-            ((Gtk.ScrolledWindow)gxml["SourceContainer"]).Add(sourceViewLoadingVP);
-            
             sourceView = new SourceView();
-            sourceView.ButtonPressEvent += OnSourceViewButtonPressEvent;
-            sourceView.Sensitive = false;
+            sourceView.SourceDoubleClicked += delegate {
+                playlistModel.PlayingIter = TreeIter.Zero;
+                playlistModel.Advance();
+                playlistView.UpdateView();
+            };
+            
+            sourceView.Sensitive = true;
+            sourceView.Show();
+            
             SourceManager.ActiveSourceChanged += OnSourceManagerActiveSourceChanged;
             SourceManager.SourceUpdated += OnSourceManagerSourceUpdated;
             SourceManager.SourceViewChanged += OnSourceManagerSourceViewChanged;
             SourceManager.SourceTrackAdded += OnSourceTrackAdded;
             SourceManager.SourceTrackRemoved += OnSourceTrackRemoved;
+            
+            ((Gtk.ScrolledWindow)gxml["SourceContainer"]).Add(sourceView);
 
             InterfaceElements.MainContainer = gxml["MainContainer"] as VBox;
 
@@ -455,6 +462,8 @@ namespace Banshee
             };
             searchEntry.Changed += OnSimpleSearch;
             searchEntry.Show();
+            gxml["SearchLabel"].Sensitive = true;
+            searchEntry.Sensitive = true;
             InterfaceElements.SearchEntry = searchEntry;
             ((HBox)gxml["PlaylistHeaderBox"]).PackStart(searchEntry, false, false, 0);
                 
@@ -557,21 +566,8 @@ namespace Banshee
             }
         }
 
-        private void LoadSourceView()
-        {        
-            sourceView.Sensitive = true;
-            ((Gtk.ScrolledWindow)gxml["SourceContainer"]).Remove(sourceViewLoadingVP);
-            ((Gtk.ScrolledWindow)gxml["SourceContainer"]).Add(sourceView);
-            sourceView.Show();
-            
-            gxml["SearchLabel"].Sensitive = true;
-            searchEntry.Sensitive = true;
-        }
-        
         private void OnLibraryReloaded(object o, EventArgs args)
         {
-            LoadSourceView();
-            
             if(LocalQueueSource.Instance.Count > 0) {
                 SourceManager.SetActiveSource(LocalQueueSource.Instance);
             } else if(Globals.ArgumentQueue.Contains("audio-cd")) {
@@ -596,15 +592,7 @@ namespace Banshee
                 });
             }
         }
-        
-        private bool PromptForImportTimeout()
-        {
-            LoadSourceView();
-            PromptForImport();
-            
-            return false;
-        }
-        
+
         // ---- Misc. Utility Routines ----
       
         private bool OnShutdownRequested()
@@ -676,6 +664,22 @@ namespace Banshee
             Globals.Configuration.Set(GConfKeys.WindowHeight, height);
         }
         
+        private static Gdk.ModifierType [] important_modifiers = new Gdk.ModifierType [] {
+            Gdk.ModifierType.ControlMask,
+            Gdk.ModifierType.ShiftMask
+        };
+            
+        private static bool NoImportantModifiersAreSet()
+        {
+            Gdk.ModifierType state = (Gtk.Global.CurrentEvent as Gdk.EventKey).State;
+            foreach(Gdk.ModifierType modifier in important_modifiers) {
+                if((state & modifier) == modifier) {
+                    return false;
+                }
+            }
+            return true;
+        }
+        
         private bool accel_group_active = true;
         
         [GLib.ConnectBefore]
@@ -684,7 +688,7 @@ namespace Banshee
             bool handled = false;
             
             if(WindowPlayer.Focus is Entry && Gtk.Global.CurrentEvent is Gdk.EventKey &&
-                (Gtk.Global.CurrentEvent as Gdk.EventKey).State == Gdk.ModifierType.None) {
+                NoImportantModifiersAreSet()) {
                 if(accel_group_active) {
                     WindowPlayer.RemoveAccelGroup(Globals.ActionManager.UI.AccelGroup);
                     accel_group_active = false;
@@ -786,6 +790,7 @@ namespace Banshee
                 case PlayerEngineState.Loaded:
                     incrementedCurrentSongPlayCount = false;
                     seek_slider.Duration = PlayerEngineCore.CurrentTrack.Duration.TotalSeconds;
+                    Globals.ActionManager["JumpToPlayingAction"].Visible = true;
                     UpdateMetaDisplay();
                     playlistView.QueueDraw();
                     
@@ -802,6 +807,7 @@ namespace Banshee
                     break;
                 case PlayerEngineState.Idle:
                     Globals.ActionManager.UpdateAction("PlayPauseAction", Catalog.GetString("Play"), "media-playback-start");
+                    Globals.ActionManager["JumpToPlayingAction"].Visible = false;
                     seek_slider.SetIdle();
                     trackInfoHeader.SetIdle();
                     
@@ -855,8 +861,16 @@ namespace Banshee
                     stream_position_label.BufferingProgress = args.BufferingPercent;
                     break;
                 case PlayerEngineEvent.Error:
-                    LogCore.Instance.PushError(Catalog.GetString("Playback Error"), args.Message);
-                    UpdateMetaDisplay();
+                    RepeatMode old_repeat_mode = playlistModel.Repeat;
+                    bool old_shuffle_mode = playlistModel.Shuffle;
+                    
+                    playlistModel.Repeat = RepeatMode.ErrorHalt;
+                    playlistModel.Shuffle = false;
+                    
+                    Next();
+                    
+                    playlistModel.Repeat = old_repeat_mode;
+                    playlistModel.Shuffle = old_shuffle_mode;
                     break;
                 case PlayerEngineEvent.TrackInfoUpdated:
                     UpdateMetaDisplay();
@@ -1017,20 +1031,64 @@ namespace Banshee
         private void OnSourceTrackAdded(object o, TrackEventArgs args)
         {
             if(SourceManager.ActiveSource == o) {
-                if(searchEntry.IsQueryAvailable && !DoesTrackMatchSearch(args.Track)) {
-                    return;
+                // We only want to update the playlist view once
+                if (args.Track != null) {
+                    AddTrackToModel(args.Track, (args.Tracks == null || args.Tracks.Count == 0));
+                }
+
+                if (args.Tracks != null) {
+                    if (args.Tracks.Count > 3) {
+                        playlistModel.ClearSortOrder();
+                    }
+
+                    int i = 1, last = args.Tracks.Count;
+                    foreach (TrackInfo track in args.Tracks) {
+                        AddTrackToModel(track, i++ == last);
+                    }
+
+                    if (args.Tracks.Count > 3) {
+                        playlistModel.RestoreSortOrder();
+                    }
                 }
-                
-                playlistModel.AddTrack(args.Track);
             }
         }
+
+        private void AddTrackToModel(TrackInfo track, bool update)
+        {
+            if(searchEntry.IsQueryAvailable && !DoesTrackMatchSearch(track)) {
+                return;
+            }
+            
+            playlistModel.AddTrack(track, update);
+        }
         
         private void OnSourceTrackRemoved(object o, TrackEventArgs args)
         {
             if(SourceManager.ActiveSource == o) {
-                playlistModel.RemoveTrack(args.Track);
+                if (args.Track != null) {
+                    RemoveTrackFromModel(args.Track);
+                }
+
+                if (args.Tracks != null) {
+                    if (args.Tracks.Count > 3) {
+                        playlistModel.ClearSortOrder();
+                    }
+
+                    foreach (TrackInfo track in args.Tracks) {
+                        RemoveTrackFromModel(track);
+                    }
+
+                    if (args.Tracks.Count > 3) {
+                        playlistModel.RestoreSortOrder();
+                    }
+                }
             }
         }
+
+        private void RemoveTrackFromModel(TrackInfo track)
+        {
+            playlistModel.RemoveTrack(track);
+        }
         
         private void UpdateViewName(Source source)
         {
@@ -1039,40 +1097,7 @@ namespace Banshee
         
         private void SensitizeActions(Source source)
         {
-            Globals.ActionManager["WriteCDAction"].Visible = !(source is AudioCdSource);
-            Globals.ActionManager["WriteCDAction"].Sensitive = source is Banshee.Burner.BurnerSource;
-            
-            Globals.ActionManager.AudioCdActions.Visible = source is AudioCdSource;
-            Globals.ActionManager["RenameSourceAction"].Visible = source.CanRename;
-            Globals.ActionManager["UnmapSourceAction"].Visible = source.CanUnmap;
-            Globals.ActionManager.DapActions.Visible = source is DapSource;
-            Globals.ActionManager["SelectedSourcePropertiesAction"].Sensitive = source.HasProperties;
-            
-            if(source is IImportSource) {
-                Globals.ActionManager["ImportSourceAction"].Visible = source is IImportSource;
-                Globals.ActionManager["ImportSourceAction"].Label = Catalog.GetString("Import") + " '" + 
-                    source.Name + "'";
-            } else {
-                Globals.ActionManager["ImportSourceAction"].Visible = false;
-            }
-            
-            if(source is DapSource) {
-                DapSource dapSource = source as DapSource;
-                if (dapSource.Device.CanSynchronize) {
-                    Globals.ActionManager["SyncDapAction"].Sensitive = !dapSource.Device.IsReadOnly;
-                    Globals.ActionManager.SetActionLabel("SyncDapAction", String.Format("{0} {1}",
-                        Catalog.GetString("Synchronize"), dapSource.Device.GenericName));
-                } else {
-                    Globals.ActionManager["SyncDapAction"].Visible = false;
-                }
-            }
-
-            Globals.ActionManager["RenameSourceAction"].Label = String.Format (
-                    Catalog.GetString("Rename {0}"), source.GenericName
-            );
-
-            Globals.ActionManager["UnmapSourceAction"].Label = source.UnmapLabel;
-            Globals.ActionManager["UnmapSourceAction"].StockId = source.UnmapIcon;
+            SourceManager.SensitizeActions(source);
         }
      
         // Called when SourceManager emits an ActiveSourceChanged event.
@@ -1086,6 +1111,11 @@ namespace Banshee
             searchEntry.Ready = false;
             searchEntry.CancelSearch();
             
+            if(source.FilterQuery != null && source.FilterField != null) {
+                searchEntry.Query = source.FilterQuery;
+                searchEntry.Field = source.FilterField;
+            }
+            
             if(source is DapSource) {
                 DapSource dap_source = source as DapSource;
                 UpdateDapDiskUsageBar(dap_source);
@@ -1188,33 +1218,36 @@ namespace Banshee
         private void UpdateStatusBar()
         {
             long count = playlistModel.Count();
+            
+            if(count == 0 && SourceManager.ActiveSource == null) {
+                LabelStatusBar.Text = Branding.ApplicationLongName;
+                return;
+            } else if(count == 0) {
+                LabelStatusBar.Text = String.Empty;
+                return;
+            } 
+            
             TimeSpan span = playlistModel.TotalDuration;       
-            string timeDisp = String.Empty;
+            StringBuilder builder = new StringBuilder();
+            
+            builder.AppendFormat(Catalog.GetPluralString("{0} Item", "{0} Items", (int)count), count);
+            builder.Append(", ");
             
             if(span.Days > 0) {
-                timeDisp = String.Format(Catalog.GetPluralString("{0} day", "{0} days", span.Days), 
-                    span.Days) + " ";
+                builder.AppendFormat(Catalog.GetPluralString("{0} day", "{0} days", span.Days), span.Days);
+                builder.Append(", ");
             }
             
-            if(span.Days > 0 || span.Hours > 0) {
-                timeDisp += String.Format("{0}:{1}:{2}", span.Hours, span.Minutes.ToString("00"), 
-                    span.Seconds.ToString("00"));
-            } else {   
-                timeDisp += String.Format("{0}:{1}", span.Minutes, span.Seconds.ToString("00"));
+            if(span.Hours > 0) {
+                builder.AppendFormat(Catalog.GetPluralString("{0} hour", "{0} hours", span.Hours), span.Hours);
+                builder.Append(", ");
             }
             
-            ThreadAssist.ProxyToMain(delegate {
-                if(count == 0 && SourceManager.ActiveSource == null) {
-                    LabelStatusBar.Text = Branding.ApplicationLongName;
-                } else if(count == 0) {
-                    LabelStatusBar.Text = String.Empty;
-                } else {
-                    string text = String.Format(Catalog.GetPluralString("{0} Item", "{0} Items", 
-                        (int)count), count) + ", ";
-                    text += String.Format(Catalog.GetString("{0} Total Play Time"), timeDisp);
-                    LabelStatusBar.Text = text;
-                }
-            });
+            builder.AppendFormat(Catalog.GetPluralString("{0} minute", "{0} minutes", span.Minutes), span.Minutes);
+            builder.Append(", ");
+            builder.AppendFormat(Catalog.GetPluralString("{0} second", "{0} seconds", span.Seconds), span.Seconds);
+            
+            LabelStatusBar.Text = builder.ToString();
         }
         
         private void OnLogCoreUpdated(object o, LogCoreUpdatedArgs args)
@@ -1255,47 +1288,7 @@ namespace Banshee
             
             dialog.ShowAll();
         }
-     
-        [GLib.ConnectBefore]
-        private void OnSourceViewButtonPressEvent(object o, ButtonPressEventArgs args)
-        {       
-            TreePath path;
-            if(!sourceView.GetPathAtPos((int)args.Event.X, (int)args.Event.Y, out path)) {
-                args.RetVal = true; 
-                return;
-            }
 
-            Source source = sourceView.GetSource(path);
-
-            if(args.Event.Button == 1 && args.Event.Type == EventType.TwoButtonPress) {
-                if(SourceManager.ActiveSource != source) {
-                    SourceManager.SetActiveSource(source);
-                }
- 
-                playlistModel.PlayingIter = TreeIter.Zero;
-                playlistModel.Advance();
-                playlistView.UpdateView();
-
-                args.RetVal = false;
-            } else if(args.Event.Button == 3) {
-                sourceView.HighlightPath(path);
-
-                SensitizeActions(source);
-
-                string group_name = source.ActionPath == null ? "/SourceMenu" : source.ActionPath;
-                Menu source_menu = Globals.ActionManager.GetWidget(group_name) as Menu;
-                source_menu.SelectionDone += delegate {
-                    SensitizeActions(SourceManager.ActiveSource);
-                    sourceView.ResetHighlight();
-                };
-            
-                source_menu.Popup(null, null, null, 0, args.Event.Time);
-                source_menu.Show();
-            
-                args.RetVal = true;
-            }
-        }
-      
         private bool DoesTrackMatchSearch(TrackInfo ti)
         {
             if(!searchEntry.IsQueryAvailable) {
@@ -1376,6 +1369,9 @@ namespace Banshee
         
         private void OnSimpleSearch(object o, EventArgs args)
         {
+            SourceManager.ActiveSource.FilterField = searchEntry.Field;
+            SourceManager.ActiveSource.FilterQuery = searchEntry.Query;
+        
             if(SourceManager.ActiveSource.HandlesSearch) {
                 return;
             }
@@ -1870,7 +1866,7 @@ namespace Banshee
                     DialogFlags.DestroyWithParent, MessageType.Warning,
                     ButtonsType.None,
                     header, message);
-                md.AddButton("gtk-cancel", ResponseType.No, true);
+                md.AddButton("gtk-cancel", ResponseType.No, false);
                 md.AddButton(button_label, ResponseType.Yes, false);
                 
                 try {
@@ -1898,7 +1894,7 @@ namespace Banshee
                     if(deleteFromFileSystem) {
                         DeleteSong(track);
                     }
-                } catch (UnauthorizedAccessException e) {
+                } catch (UnauthorizedAccessException) {
                     string header = Catalog.GetString ("Delete songs from drive");
                     string msg = String.Format (Catalog.GetString ("You do not have the required permissions to delete '{0}'"), track.Uri.LocalPath);
                     HigMessageDialog error = new HigMessageDialog (WindowPlayer,
@@ -2043,7 +2039,7 @@ namespace Banshee
                     dapSource.Device.GenericName) + (dapSource.Device.GenericName.ToLower() == "ipod" ?
                     ("\n\n" + 
                     Catalog.GetString("<b>Warning:</b> Actions will alter or erase existing iPod contents and " +
-                    "may cause incompatability with iTunes!")) : ""),
+                    "may cause incompatibility with iTunes!")) : ""),
                 Catalog.GetString("Synchronize Library"));
             
             md.AddButton(Catalog.GetString("Save Manual Changes"), Gtk.ResponseType.Apply, true);
@@ -2074,6 +2070,40 @@ namespace Banshee
         
         // --- Edit Menu ---
         
+        private List<TrackInfo> clipboard_tracks = new List<TrackInfo>();
+        
+        private void OnCopySongsAction(object o, EventArgs args)
+        {
+            clipboard_tracks.Clear();
+            
+            foreach(TrackInfo track in playlistView.SelectedTrackInfoMultiple) {
+                clipboard_tracks.Add(track);
+            }
+            
+            Clipboard clipboard = Clipboard.Get(Gdk.Selection.Clipboard);
+            clipboard.SetWithData(nautilus_file_copy_entries, OnGetClipboard, OnClearClipboard);   
+        }
+        
+        private void OnGetClipboard(Clipboard clipboard, SelectionData selection, uint info)
+        {
+            StringBuilder uris = new StringBuilder();
+            
+            uris.Append("copy\n");
+            
+            foreach(TrackInfo track in clipboard_tracks) {
+                uris.Append(track.Uri);
+                uris.Append("\n");
+            }
+            
+            byte [] raw_selection_data = Encoding.UTF8.GetBytes(uris.ToString());
+            selection.Set(selection.Target, 8, raw_selection_data, raw_selection_data.Length);
+        }
+
+        private void OnClearClipboard(Clipboard clipboard)
+        {   
+            clipboard_tracks.Clear();
+        }
+
         private void OnRemoveSongsAction(object o, EventArgs args)
         {
             RemoveSongs(false);
