@@ -66,13 +66,16 @@ namespace Banshee
         };
             
         private ArrayList columns;
-        PlaylistModel model;
+        private PlaylistModel model;
         
-        PlaylistColumnChooserDialog columnChooser;
-        Pixbuf nowPlayingPixbuf;
-        Pixbuf songDrmedPixbuf;
-        Pixbuf ripColumnPixbuf;
-        Banshee.Gui.CellRendererRating rating_renderer;
+        private PlaylistColumnChooserDialog columnChooser;
+        private Pixbuf ripColumnPixbuf;
+        private Banshee.Gui.CellRendererRating rating_renderer;
+
+        private Pixbuf now_playing_pixbuf;
+        private Pixbuf drm_pixbuf;
+        private Pixbuf resource_not_found_pixbuf;
+        private Pixbuf unknown_error_pixbuf;
 
         public TreeViewColumn RipColumn;
         public PlaylistColumn RatingColumn;
@@ -157,9 +160,12 @@ namespace Banshee
             playIndColumn.Reorderable = false;
             playIndColumn.Widget = playIndImg;
             
-            nowPlayingPixbuf = IconThemeUtils.LoadIcon(16, "media-playback-start", 
+            now_playing_pixbuf = IconThemeUtils.LoadIcon(16, "media-playback-start", 
                 Stock.MediaPlay, "now-playing-arrow");
-            songDrmedPixbuf = Gdk.Pixbuf.LoadFromResource("song-drm.png");
+            drm_pixbuf = IconThemeUtils.LoadIcon(16, "emblem-readonly", "emblem-important", Stock.DialogError);
+            resource_not_found_pixbuf = IconThemeUtils.LoadIcon(16, "emblem-unreadable", Stock.DialogError);
+            unknown_error_pixbuf = IconThemeUtils.LoadIcon(16, "dialog-error", Stock.DialogError);
+            
             ripColumnPixbuf = Gdk.Pixbuf.LoadFromResource("cd-action-rip-16.png");
             
             CellRendererPixbuf indRenderer = new CellRendererPixbuf();
@@ -248,6 +254,8 @@ namespace Banshee
             return prev != null;
         }
 
+        /* Model Sorting Comparisons */
+
         private int StringFieldCompare(string a, string b)
         {
             if(a != null)
@@ -263,94 +271,72 @@ namespace Banshee
         {
             return a < b ? -1 : (a == b ? 0 : 1);
         }
-            
-        public int TrackTreeIterCompareFunc(TreeModel _model, TreeIter a,
-            TreeIter b)
-        {
+        
+        private int TrackBaseTreeIterCompareFunc(TreeModel _model, TreeIter a, TreeIter b)
+        {   
             return LongFieldCompare((long)model.IterTrackInfo(a).TrackNumber,
                 (long)model.IterTrackInfo(b).TrackNumber);
         }
             
-        public int ArtistTreeIterCompareFunc(TreeModel _model, TreeIter a, 
-            TreeIter b)
+        private int TrackTreeIterCompareFunc(TreeModel _model, TreeIter a, TreeIter b)
         {
-            int v = StringFieldCompare(model.IterTrackInfo(a).Artist, 
-                model.IterTrackInfo(b).Artist);
+            int v = ArtistTreeIterCompareFunc(_model, a, b);
+            return v != 0 ? v : TrackBaseTreeIterCompareFunc(_model, a, b);
+        }
             
-            if(v != 0)
-                return v;
-            
-            return AlbumTreeIterCompareFunc(_model, a, b);
+        private int ArtistTreeIterCompareFunc(TreeModel _model, TreeIter a, TreeIter b)
+        {
+            int v = StringFieldCompare(model.IterTrackInfo(a).Artist, model.IterTrackInfo(b).Artist);
+            return v != 0 ? v : AlbumTreeIterCompareFunc(_model, a, b);
         }
         
-        public int TitleTreeIterCompareFunc(TreeModel _model, TreeIter a, 
-            TreeIter b)
+        private int TitleTreeIterCompareFunc(TreeModel _model, TreeIter a, TreeIter b)
         {
-            return StringFieldCompare(model.IterTrackInfo(a).Title, 
-                model.IterTrackInfo(b).Title);
+            return StringFieldCompare(model.IterTrackInfo(a).Title, model.IterTrackInfo(b).Title);
         }
         
-        public int AlbumTreeIterCompareFunc(TreeModel _model, TreeIter a, 
-            TreeIter b)
+        private int AlbumTreeIterCompareFunc(TreeModel _model, TreeIter a, TreeIter b)
         {
-            int v = StringFieldCompare(model.IterTrackInfo(a).Album, 
-                model.IterTrackInfo(b).Album);
-
-            if(v != 0)
-                return v;
-
-            return TrackTreeIterCompareFunc(_model, a, b);
+            int v = StringFieldCompare(model.IterTrackInfo(a).Album, model.IterTrackInfo(b).Album);
+            return v != 0 ? v : TrackBaseTreeIterCompareFunc(_model, a, b);
         }
         
-        public int GenreTreeIterCompareFunc(TreeModel _model, TreeIter a,
-            TreeIter b)
+        private int GenreTreeIterCompareFunc(TreeModel _model, TreeIter a, TreeIter b)
         {
-            return StringFieldCompare(model.IterTrackInfo(a).Genre, 
-                model.IterTrackInfo(b).Genre);
+            return StringFieldCompare(model.IterTrackInfo(a).Genre, model.IterTrackInfo(b).Genre);
         }
         
-        public int YearTreeIterCompareFunc(TreeModel _model, TreeIter a,
-            TreeIter b)
+        private int YearTreeIterCompareFunc(TreeModel _model, TreeIter a, TreeIter b)
         {
-            return LongFieldCompare(model.IterTrackInfo(a).Year, 
-                model.IterTrackInfo(b).Year);
+            return LongFieldCompare(model.IterTrackInfo(a).Year, model.IterTrackInfo(b).Year);
         }
         
-        public int TimeTreeIterCompareFunc(TreeModel _model, TreeIter a,
-            TreeIter b)
+        private int TimeTreeIterCompareFunc(TreeModel _model, TreeIter a, TreeIter b)
         {
-            //return LongFieldCompare(model.IterTrackInfo(a).Duration,
-            //    model.IterTrackInfo(b).Duration);
-                
             return model.IterTrackInfo(a).Duration.CompareTo(model.IterTrackInfo(b).Duration);
         }
         
-        public int RatingTreeIterCompareFunc(TreeModel _model, TreeIter a,
-            TreeIter b)
+        private int RatingTreeIterCompareFunc(TreeModel _model, TreeIter a, TreeIter b)
         {
-            return LongFieldCompare((long)model.IterTrackInfo(a).Rating,
-                (long)model.IterTrackInfo(b).Rating);
+            return LongFieldCompare((long)model.IterTrackInfo(a).Rating, (long)model.IterTrackInfo(b).Rating);
         }
         
-        public int PlayCountTreeIterCompareFunc(TreeModel _model, TreeIter a,
-            TreeIter b)
+        public int PlayCountTreeIterCompareFunc(TreeModel _model, TreeIter a, TreeIter b)
         {
-            return LongFieldCompare((long)model.IterTrackInfo(a).PlayCount,
-                (long)model.IterTrackInfo(b).PlayCount);
+            return LongFieldCompare((long)model.IterTrackInfo(a).PlayCount, (long)model.IterTrackInfo(b).PlayCount);
         }
         
-        public int LastPlayedTreeIterCompareFunc(TreeModel _model, TreeIter a,
-            TreeIter b)
+        private int LastPlayedTreeIterCompareFunc(TreeModel _model, TreeIter a, TreeIter b)
         {
-            return DateTime.Compare(model.IterTrackInfo(a).LastPlayed,
-                model.IterTrackInfo(b).LastPlayed);
+            return DateTime.Compare(model.IterTrackInfo(a).LastPlayed, model.IterTrackInfo(b).LastPlayed);
         }
         
-        public int DefaultTreeIterCompareFunc(TreeModel _model, TreeIter a, 
-            TreeIter b)
+        private int DefaultTreeIterCompareFunc(TreeModel _model, TreeIter a, TreeIter b)
         {
             return 0;
         }
+        
+        /* ----- */
             
         public TrackInfo IterTrackInfo(TreeIter iter)
         {
@@ -383,16 +369,40 @@ namespace Banshee
                 : (int)Pango.Weight.Normal;
             
             renderer.Foreground = null;
-            
             renderer.Sensitive = true;
             
             TrackInfo ti = model.IterTrackInfo(iter);
             if(ti == null) {
                 return;
             }
-          
-            if(ti is AudioCdTrackInfo) {
-                renderer.Sensitive = ti.CanPlay; 
+            
+            renderer.Sensitive = ti.CanPlay && ti.PlaybackError == TrackPlaybackError.None;
+        }
+        
+        private void SetTrackPixbuf(CellRendererPixbuf renderer, TrackInfo track, bool nowPlaying)
+        {
+            if(nowPlaying) {
+                renderer.Pixbuf = now_playing_pixbuf;
+                return;
+            } else if(track is AudioCdTrackInfo) {
+                renderer.Pixbuf = null;
+                return;
+            }
+            
+            switch(track.PlaybackError) {
+                case TrackPlaybackError.ResourceNotFound:
+                    renderer.Pixbuf = resource_not_found_pixbuf;
+                    break;
+                case TrackPlaybackError.Drm:
+                    renderer.Pixbuf = drm_pixbuf;
+                    break;
+                case TrackPlaybackError.Unknown:
+                case TrackPlaybackError.CodecNotFound:
+                    renderer.Pixbuf = unknown_error_pixbuf;
+                    break;
+                default:
+                    renderer.Pixbuf = null;
+                    break;
             }
         }
         
@@ -404,12 +414,7 @@ namespace Banshee
             
             if(PlayerEngineCore.CurrentTrack == null) {
                 model.PlayingIter = TreeIter.Zero;
-                if(ti != null && !(ti is AudioCdTrackInfo)) {
-                    renderer.Pixbuf = ti.CanPlay ? null : songDrmedPixbuf;
-                } else {
-                    renderer.Pixbuf = null;
-                }
-                
+                SetTrackPixbuf(renderer, ti, false);
                 return;
             }
         
@@ -421,14 +426,7 @@ namespace Banshee
                     same_track = PlayerEngineCore.CurrentTrack == ti;
                 }
                 
-                if(same_track) {
-                    renderer.Pixbuf = nowPlayingPixbuf;
-                    model.PlayingIter = iter;
-                } else if(ti is AudioCdTrackInfo) {
-                    renderer.Pixbuf = null;
-                } else {
-                    renderer.Pixbuf = ti.CanPlay ? null : songDrmedPixbuf;
-                }
+                SetTrackPixbuf(renderer, ti, same_track);
             } else {
                 renderer.Pixbuf = null;
             }
@@ -479,7 +477,29 @@ namespace Banshee
                 return;
             }
             
-            SetRendererAttributes((CellRendererText)cell, ti.Title, iter);
+            string suffix = null;
+            
+            switch(ti.PlaybackError) {
+                case TrackPlaybackError.ResourceNotFound:
+                    suffix = Catalog.GetString("Missing");
+                    break;
+                case TrackPlaybackError.Drm:
+                    suffix = "DRM";
+                    break;
+                case TrackPlaybackError.CodecNotFound:
+                    suffix = Catalog.GetString("No Codec");
+                    break;
+                case TrackPlaybackError.Unknown:
+                    suffix = Catalog.GetString("Unknown Error");
+                    break;
+                default:
+                    break;
+            }
+            
+            SetRendererAttributes((CellRendererText)cell, suffix == null 
+                ? ti.Title
+                : String.Format("({0}) {1}", suffix, ti.Title),
+                iter);
         }
         
         protected void TrackCellAlbum(TreeViewColumn tree_column,
