@@ -1,7 +1,8 @@
 /***************************************************************************
-    copyright            : (C) 2006 by Brian Nickel
+    copyright            : (C) 2005 by Brian Nickel
+                         : (C) 2006 Novell, Inc.
     email                : brian.nickel@gmail.com
-    based on             : audioproperties.cpp from TagLib
+                         : Aaron Bockover <abockover@novell.com>
  ***************************************************************************/
 
 /***************************************************************************
@@ -42,32 +43,10 @@ namespace TagLib
    }
    
    public abstract class File
-   {
-      private static Hashtable file_types = new Hashtable();
-      
+   {  
       public delegate IFileAbstraction FileAbstractionCreator (string path);
       public delegate File             FileTypeResolver       (string path, AudioProperties.ReadStyle style);
-      
-      static File()
-      {
-         System.Reflection.Assembly assembly = System.Reflection.Assembly.GetExecutingAssembly();
-
-         foreach(Type type in assembly.GetTypes()) {
-            if(!type.IsSubclassOf(typeof(File))) {
-               continue;
-            }
-            
-            Attribute [] attrs = Attribute.GetCustomAttributes(type, typeof(SupportedMimeType));
-            if(attrs == null || attrs.Length == 0) {
-               continue;
-            }
-            
-            foreach(SupportedMimeType attr in attrs) {
-               file_types.Add(attr.MimeType, type);
-            } 
-         }
-      }
-      
+         
       public enum AccessMode
       {
          Read,
@@ -563,12 +542,12 @@ namespace TagLib
             mimetype = "taglib/" + ext.ToLower();
          }
  
-         Type file_type = file_types[mimetype] as Type;
-         if(file_type == null)
-         {
+         if(!FileTypes.AvailableTypes.ContainsKey(mimetype)) {
             throw new UnsupportedFormatException(String.Format("{0} ({1})", path, mimetype));
          }
          
+         Type file_type = FileTypes.AvailableTypes[mimetype];
+                 
          try {
             File file = (File)Activator.CreateInstance(file_type, new object [] { path, style });
             file.MimeType = mimetype;
